/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   SELOR - Servidor de Emails Livre Orientado a Regras                           
   Programado por Lucas Priori                                                   
   Versao: 1.2                                                                   
                                                                                 
   Linux - make linux                                                            
       or  make linux_nocap                                                      
   FreeBSD - gmake freebsd                                                       
                                                                                 
  *******************************************************************            
  Selor e um agente MTA para troca de emails via protocolo SMTP                 
  Copyright (C) 2012,2013,2014,2015  Lucas Priori                                
                                                                                 
  This program is free software; you can redistribute it and/or                  
  modify it under the terms of the GNU General Public License                    
  as published by the Free Software Foundation; either version 2                 
  of the License, or (at your option) any later version.                         
                                                                                 
  This program is distributed in the hope that it will be useful,                
  but WITHOUT ANY WARRANTY; without even the implied warranty of                 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  
  GNU General Public License for more details.                                   
                                                                                 
  You should have received a copy of the GNU General Public License              
  along with this program; if not, write to the Free Software                    
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/


#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<errno.h>
#include<time.h>
#include<netinet/in.h>
#include<sys/socket.h>
#include<arpa/inet.h>
#include<pwd.h>
#include<unistd.h>
#include<netdb.h>

#define VERSAO "v1.0"
#define TAM_buffer 2048
#define MAX_rcpts 100
#define MAX_tam_email 200
#define MAX_tam_subject 200
#define MAX_tam_host 200
#define MAX_tam_user 100
#define MAX_tam_pass 50
#define MAX_tam_date 50
#define MAX_anexos 50
#define CAM_tmp "/tmp/"
#define DEF_timeout 30


// -- Variaveis Globais de Socket --
struct sockaddr_in rede_remota;
int  socket_remoto;
int numero=0;
struct hostent *hostname;
// -- Variaveis Globais --
char buffer[TAM_buffer+1];
char linha[TAM_buffer+1];
char rcpts[MAX_rcpts][MAX_tam_email];
char from[MAX_tam_email];
char subject[MAX_tam_subject];
char ip[16]="127.0.0.1";
char host[MAX_tam_host];
char user[MAX_tam_user];
char pass[MAX_tam_pass];
char date[MAX_tam_date];
char op_v=0;
char op_sf[TAM_buffer+1];
char *op_a[MAX_anexos];
char modo_anexo=0;
char str_fuso[6];
char boundary[100]={0};
int from_ativo=0;
int to_ativo=0;
int subject_ativo=0;
int date_ativo=0;
int from_rest=0;
int to_rest=0;
int subject_rest=0;
int total_anexos=0;
unsigned int tam_header=0;
unsigned short port=25;
unsigned int num_rcpts=0;

//TEMPO
struct servs_tempo{
 char dia_semana[3];
 char fim1;
 char mes[3];
 char fim2;
 char dia[2];
 char fim3;
 char hora[2];
 char fim4;
 char minuto[2];
 char fim5;
 char segundo[2];
 char fim6;
 char ano[4];
 char fim7;
}infotempo;
struct tm *timeinfo;
time_t tempo;

char tab_base64[65]=
{ 'A','B','C','D','E','F','G','H','I','J','K','L','M',
  'N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
  'a','b','c','d','e','f','g','h','i','j','k','l','m',
  'n','o','p','q','r','s','t','u','v','w','x','y','z',
  '0','1','2','3','4','5','6','7','8','9','+','/', 0
};


//---------------------------------------------------
//Funcao semelhante a strncpy, porem, para quando    
// encontra o fim da string origem e retorna o numero
// de bytes copiados                                 
// Retorna o numero de bytes copiados                
int selor_strncpy(char *destino, char *origem, int num_bytes){
 int ret_val=0;
 
 if(!num_bytes)return 0;
 #ifdef A64
  asm(
   "mov  %0,%%rdi          \n"
   "mov  %1,%%rsi          \n"
   "mov  %%rcx, %%rdx      \n"
  "STRNCPY_Volta:          \n"
   "lodsb                  \n"
   "stosb                  \n"
   "test %%al,%%al         \n"
   "jz  STRNCPY_Sair       \n"
   "loop STRNCPY_Volta     \n"
   "movb  $0, (%%rdi)      \n"
  "STRNCPY_Sair:           \n" 
   "sub  %%rcx, %%rdx      \n"
   "movw %%dx, %3          \n"
   :
   :"m"(destino),"m"(origem),"c"(num_bytes),"m"(ret_val)
   :"rdi","rsi","rdx"
  );
 #else
   asm(
   "mov  %0,%%edi          \n"
   "mov  %1,%%esi          \n"
   "mov  %%ecx, %%edx      \n"
  "STRNCPY_Volta:          \n"
   "lodsb                  \n"
   "stosb                  \n"
   "test %%al,%%al         \n"
   "jz  STRNCPY_Sair       \n"
   "loop STRNCPY_Volta     \n"
   "movb  $0, (%%edi)      \n"
  "STRNCPY_Sair:           \n" 
   "sub  %%ecx, %%edx      \n"
   "movw %%dx, %3          \n"
   :
   :"m"(destino),"m"(origem),"c"(num_bytes),"m"(ret_val)
   :"edi","esi","edx"
  );
 #endif
 return ret_val;
}

//---------------------------------------------------
//Funcao para comparar duas strings desconciderando  
//a diferena entre maisculas e minsculas          
//Retornos:                                          
//0 = strings so iguais                             
//1 = strings so diferentes                         
int cmp_str(char *string1, char *string2, int num_bytes){
 
 if(!num_bytes){
  if(string1[0]==string2[0])return 0;
  return 1;
 }
 int ret_val=0;
 
 #ifdef A64
  asm(
   "mov  %0,%%rdi           \n"
   "mov  %1,%%rsi           \n"
  "CMP_STR_proxbyte:        \n"
   "movb (%%rsi),%%al       \n"
   "cmpb (%%rdi),%%al       \n"
   "jne  CMP_STR_trocar     \n"
  "CMP_STR_incr:            \n"
   "inc  %%rsi              \n"
   "inc  %%rdi              \n"
   "loop CMP_STR_proxbyte   \n"
   "jmp  CMP_STR_term       \n"
  "CMP_STR_trocar:          \n"
   "xorb $32,%%al           \n"
   "cmpb (%%rdi),%%al       \n"
   "je   CMP_STR_incr       \n"
  "CMP_STR_term:            \n"
   "testw %%cx,%%cx         \n"
   "jz   CMP_STR_fim        \n"
   "movw $1,%%cx            \n"
  "CMP_STR_fim:             \n"
   "movw  %%cx,%3           \n"
   :
   :"m"(string1),"m"(string2),"c"(num_bytes),"m"(ret_val)
   :"rdi","rsi","rax"
  );
 #else
  asm(
   "movl %0,%%edi           \n"
   "movl %1,%%esi           \n"
  "CMP_STR_proxbyte:        \n"
   "movb (%%esi),%%al       \n"
   "cmpb (%%edi),%%al       \n"
   "jne  CMP_STR_trocar     \n"
  "CMP_STR_incr:            \n"
   "incl %%esi              \n"
   "incl %%edi              \n"
   "loop CMP_STR_proxbyte   \n"
   "jmp  CMP_STR_term       \n"
  "CMP_STR_trocar:          \n"
   "xorb $32,%%al           \n"
   "cmpb (%%edi),%%al       \n"
   "je   CMP_STR_incr       \n"
  "CMP_STR_term:            \n"
   "testw %%cx,%%cx         \n"
   "jz   CMP_STR_fim        \n"
   "movw $1,%%cx            \n"
  "CMP_STR_fim:             \n"
   "movw  %%cx,%3           \n"
   :
   :"m"(string1),"m"(string2),"c"(num_bytes),"m"(ret_val)
   :"edi","esi","eax"
  );
 #endif
 return ret_val;
}

//---------------------------------------------------
//Funo para codificar um buffer para MIME64        
void base64_codif(char *retorno, char *str_normal, int tamanho){
 char *base=tab_base64;

 #ifdef A64
  asm(
   "xor  %%edx,%%edx            \n" 
   "push %%rbx                  \n"
   "mov  $3,%%ebx               \n"
   "div  %%ebx                  \n"
   "pop  %%rbx                  \n"
   "mov  %%eax,%%ecx            \n"
   "push %%dx                   \n"
   "test %%dl,%% dl             \n"
   "jz   CODI_Pular             \n" 
   "inc  %%ecx                  \n"
  "CODI_Pular:                  \n"
   "mov  %0,%%rdi               \n"
   "mov  %1,%%rsi               \n"
  "CODI_Prox_casa:              \n"
   "push  %%rcx                 \n"
   "mov  (%%rsi),%%eax          \n"
   "ror  $8,%%ax                \n"
   "ror  $16,%%eax              \n"
   "ror  $8,%%ax                \n"
   "shr  $8,%%eax               \n"
   "mov  $4,%%cx                \n"
   "shl  $2,%%eax               \n"
   "add  $4,%%rdi               \n"
  "CODI_Prox_byte:              \n"
   "shr  $2,%%al                \n"
   "xlatb                       \n"
   "dec  %%rdi                  \n"
   "mov  %%al,(%%rdi)           \n"
   "shr  $6,%%eax               \n"
   "loop CODI_Prox_byte         \n"
   "add  $4,%%rdi               \n"
   "add  $3,%%rsi               \n"
   "pop  %%rcx                  \n"
   "loop CODI_Prox_casa         \n"
   "pop  %%dx                   \n"        // Parte que coloca o '=' no final da string 
   "test %%dl,%%dl              \n"        // gerada                                    
   "jz   CODI_Fim               \n"
   "mov  $3,%%rcx               \n"
   "sub  %%dl,%%cl              \n"
   "sub  %%rcx,%%rdi            \n"
   "movb $61,(%%rdi)            \n"
   "dec  %%cl                   \n"
   "inc  %%rdi                  \n"
   "test %%cl,%%cl              \n"
   "jz   CODI_Fim               \n"
   "movb $61,(%%rdi)            \n"
   "inc  %%rdi                  \n"
  "CODI_Fim:                    \n"
   "movb  $0,(%%rdi)             \n"
   :
   :"m"(retorno),"m"(str_normal),"b"(base),"a"(tamanho)
   :"rdi","rsi","rcx","rdx"
  );
 #else
  asm(
   "xorl %%edx,%%edx            \n" 
   "push %%ebx                  \n"
   "movl $3,%%ebx               \n"
   "divl %%ebx                  \n"
   "pop  %%ebx                  \n"
   "movl %%eax,%%ecx            \n"
   "pushw %%dx                  \n"
   "testb %%dl,%%dl             \n"
   "jz   CODI_Pular             \n" 
   "incl %%ecx                  \n"
  "CODI_Pular:                  \n"
   "movl %0,%%edi               \n"
   "movl %1,%%esi               \n"
  "CODI_Prox_casa:              \n"
   "pushl %%ecx                 \n"
   "movl (%%esi),%%eax          \n"
   "rorw $8,%%ax                \n"
   "rorl $16,%%eax              \n"
   "rorw $8,%%ax                \n"
   "shrl $8,%%eax               \n"
   "movw $4,%%cx                \n"
   "shll $2,%%eax               \n"
   "addl $4,%%edi               \n"
  "CODI_Prox_byte:              \n"
   "shrb $2,%%al                \n"
   "xlatb                       \n"
   "decl %%edi                  \n"
   "movb %%al,(%%edi)           \n"
   "shrl $6,%%eax               \n"
   "loop CODI_Prox_byte         \n"
   "addl $4,%%edi               \n"
   "addl $3,%%esi               \n"
   "popl %%ecx                  \n"
   "loop CODI_Prox_casa         \n"
   "popw %%dx                   \n"        // Parte que coloca o '=' no final da string 
   "testb %%dl,%%dl             \n"        // gerada                                    
   "jz   CODI_Fim               \n"
   "movl $3,%%ecx               \n"
   "subb %%dl,%%cl              \n"
   "subl %%ecx,%%edi            \n"
   "movb $61,(%%edi)            \n"
   "decb %%cl                   \n"
   "incl %%edi                  \n"
   "testb %%cl,%%cl             \n"
   "jz   CODI_Fim               \n"
   "movb $61,(%%edi)            \n"
   "incl %%edi                  \n"
  "CODI_Fim:                    \n"
   "movb $0,(%%edi)             \n"
   :
   :"m"(retorno),"m"(str_normal),"b"(base),"a"(tamanho)
   :"edi","esi","ecx","edx"
  );
 #endif
return ;
}



//======================================================
// Funcao para recebimento de dados via socket com prot.
// de timeout                                           
int selor_recv(int num_socket, char *buffer_local, int num_bytes, unsigned int flags){
 int recebidos;
 
 alarm(DEF_timeout);
 recebidos=recv(num_socket, buffer_local, num_bytes, flags);
 alarm(0);
 if(recebidos >= 0)buffer_local[recebidos]=0;
 
 return recebidos;
}


//======================================================
// Envia um buffer via SOCKET                           
// Retorna o numero de bytes enviados ou -1 em caso de  
// erro                                                 
int selor_send(int num_socket, char *buffer_local, int num_bytes, unsigned int flags){
 unsigned int enviados=0;
 int b_enviados=0;
 
 enviados=num_bytes;
 do{
  b_enviados=send(num_socket, buffer_local, num_bytes, flags);
  if(b_enviados < 0)return -1;
  buffer_local+=b_enviados;
  num_bytes-=b_enviados;
 }while(num_bytes && b_enviados);
 return enviados-num_bytes;
}


//======================================================
// Busca e ret. apenas o endereco de email de um texto  
//======================================================
void ret_email(char *texto, char *email_retorno){
 char *ptr_aux1;
 char *ptr_aux2;
 char byte=0;

 if(!(ptr_aux1=strchr(texto, '@'))){
  email_retorno[0]=0;
  return;
 }
 ptr_aux2=ptr_aux1+1;
 while(*ptr_aux1 != ' ' && *ptr_aux1 != '<' && ptr_aux1 > texto)ptr_aux1--;
 while(*ptr_aux2 != ' ' && *ptr_aux2 != '>' && *ptr_aux2 != 0)ptr_aux2++;
 if(*ptr_aux1 == ' ' || *ptr_aux1 == '<')ptr_aux1++;
 byte=*ptr_aux2;
 *ptr_aux2=0;
 
 selor_strncpy(email_retorno, ptr_aux1, MAX_tam_email);
 *ptr_aux2=byte;
}


//======================================================
// Pega a data e hora atual                             
//======================================================
void pegtempo(void){
 time(&tempo);
 memcpy(&infotempo,ctime(&tempo),sizeof(infotempo));
 infotempo.fim1=0;
 infotempo.fim2=0;
 infotempo.fim3=0;
 infotempo.fim4=0;
 infotempo.fim5=0;
 infotempo.fim6=0;
 infotempo.fim7=0;
 if(infotempo.dia[0] == ' ')infotempo.dia[0]='0';
}


// -------------------------------------------------------------
//                     Programa principal                       
// -------------------------------------------------------------
int main(int argc, char *argv[]){
 FILE *arq_tmp;
 char byte=0;
 char *ptr_aux1;
 char *ptr_aux2;
 char *ptr_aux3;
 char cam_arq_tmp[TAM_buffer+1];
 char header_enviado=0;
 unsigned int cont1=0;
 unsigned int cont2=0;
 unsigned int cont3=0;
 int b_gravados=0;
 int b_lidos=0;
 
 from[0]=0;
 subject[0]=0;
 rcpts[0][0]=0;
 cam_arq_tmp[0]=0;
 op_sf[0]=0;
 user[0]=0;
 pass[0]=0;
 host[0]=0;
 
 if(argc > 1){
  // -- Lendo parametros --
  for(cont1=1; cont1<argc; cont1++){
   if(!strcmp(argv[cont1], "-to") && argv[++cont1] && num_rcpts < MAX_rcpts)selor_strncpy(rcpts[num_rcpts++], argv[cont1], MAX_tam_email);
   else if(!strcmp(argv[cont1], "-f") && argv[++cont1])selor_strncpy(from, argv[cont1], MAX_tam_email);
   else if(!strcmp(argv[cont1], "-su") && argv[++cont1])selor_strncpy(subject, argv[cont1], MAX_tam_subject);
   //else if(!strcmp(argv[cont1], "-ip") && argv[++cont1])selor_strncpy(ip, argv[cont1], 16);
   else if(!strcmp(argv[cont1], "-h") && argv[++cont1])selor_strncpy(host, argv[cont1], MAX_tam_host);
   else if(!strcmp(argv[cont1], "-p") && argv[++cont1])port=atoi(argv[cont1]);
   else if(!strcmp(argv[cont1], "-sf") && argv[++cont1])selor_strncpy(op_sf, argv[cont1], TAM_buffer);
   else if(!strcmp(argv[cont1], "-v"))op_v=1;
   else if(!strcmp(argv[cont1], "--help")){
    printf("Selor_mail V %s\n", VERSAO);
    printf(" Option       | Description\n");
    printf("  -to            To force a recipient\n");
    printf("  -f             To force a sender\n");
    printf("  -su            To force a subject\n");
    printf("  -h             Specifies a hostname to send the message\n");
    printf("  -p             Specifies a port to send the message\n");
    printf("  -sf            Specifies a file that contains the mail message\n");
    printf("  -v             Enables the verbose mode\n");
    printf("  -au            To specify a username and password for authentication\n");
    printf("  -a             To specify one or more files to attach to the message\n");
    printf("                  You must specify a mail header with the message\n\n");
    return 0;
   }else if(!strcmp(argv[cont1], "-au") && argv[++cont1]){
    if((ptr_aux1=strchr(argv[cont1], ':'))){
     *ptr_aux1++=0;
     selor_strncpy(user, argv[cont1], MAX_tam_user);
     selor_strncpy(pass, ptr_aux1, MAX_tam_pass);
    }
   }else if(!strcmp(argv[cont1], "-a"))modo_anexo=1; // Lignado o modo anexo
   else if(modo_anexo){
    if(total_anexos >= MAX_anexos){
     printf("Too many attachments\n");
     return 1;
    }else op_a[total_anexos++]=argv[cont1];
   }
  }
 }
 
 time(&tempo);
 if(!op_sf[0]){
  snprintf(cam_arq_tmp, sizeof(cam_arq_tmp), "%s%lu_%d.eml", CAM_tmp, tempo, getpid());
  if(!(arq_tmp=fopen(cam_arq_tmp, "wb"))){
   if(op_v)printf("Could not create the temporary file\n");
   return 1;
  }
  cont1=0;
  buffer[0]=0;
  while((byte=fgetc(stdin)) != EOF){
   buffer[cont1++]=byte;
   if(cont1 == TAM_buffer){
    b_gravados=fwrite(&buffer, 1, cont1, arq_tmp);
    if(b_gravados != cont1){
     // Sem espaco no disco
     fclose(arq_tmp);
     remove(cam_arq_tmp);
     return 2;
    }
    cont1=0;
   }
  }
  if(cont1){
   b_gravados=fwrite(&buffer, 1, cont1, arq_tmp);
   if(b_gravados != cont1){
    // Sem espaco no disco
    fclose(arq_tmp);
    remove(cam_arq_tmp);
    return 2;
   }
  }
  fclose(arq_tmp);
  // -- Analisando o header do email --
  if(!(arq_tmp=fopen(cam_arq_tmp, "rb"))){
   if(op_v)printf("Could not open the temporary file\n");
   return 1;
  }
  from_ativo=0;
  to_ativo=0;
  subject_ativo=0;
  from_rest=MAX_tam_email;
  to_rest=MAX_tam_email;
  subject_rest=MAX_tam_subject;
  cont1=0;
  cont2=0;
  tam_header=0;
  date[0]=0;
  while(!feof(arq_tmp)){
   tam_header+=cont1;
   b_lidos=fread(buffer, 1, TAM_buffer, arq_tmp);
   if(b_lidos < 1){
    if(op_v)printf("Could not read the temporary file\n");
    return 1;
   }
   for(cont1=0; cont1<b_lidos; cont1++){
    if(buffer[cont1] == 10){
     if(!cont2){
      tam_header+=cont1;
      break;   // Fim do header
     }
     if(linha[0] != ' ' && linha[0] != 9){
      from_ativo=0;
      to_ativo=0;
      subject_ativo=0;
     }
     linha[cont2]=0;
     if(from_ativo || (!from[0] && !cmp_str(linha, "From:", 5))){
      if(!from_ativo){
       ptr_aux1=&linha[5];
       while(*ptr_aux1 == ' ')ptr_aux1++;
      }else ptr_aux1=linha;
      ptr_aux2=from+from_ativo;
      from_ativo+=selor_strncpy(ptr_aux2, ptr_aux1, from_rest);
      from_rest-=from_ativo;
      ptr_aux2=from+from_ativo;
     }else if((num_rcpts < MAX_rcpts) && (to_ativo || (!cmp_str("To:", linha, 3)) || (!cmp_str("Cc:", linha, 3)))){
      if(!to_ativo){
       ptr_aux1=&linha[3];
       while(*ptr_aux1 == ' ')ptr_aux1++;
       to_ativo=1;
      }else ptr_aux1=linha;
      ptr_aux2=ptr_aux1;
      while(ptr_aux2){
       if((ptr_aux2=strchr(ptr_aux2, '@'))){
        ptr_aux3=ptr_aux2-1; 
        while(*ptr_aux2 != ',' && *ptr_aux2 != ' ' && *ptr_aux2 != '>' && *ptr_aux2 != 0)ptr_aux2++;
        while(*ptr_aux3 != ',' && *ptr_aux3 != ' ' && *ptr_aux3 != '<' && ptr_aux3 > ptr_aux1)ptr_aux3--;
        if(*ptr_aux3 == ',' || *ptr_aux3 == ' ' || *ptr_aux3 == '<')ptr_aux3++;
        byte=*ptr_aux2;
        *ptr_aux2=0;
        selor_strncpy(rcpts[num_rcpts++], ptr_aux3, MAX_tam_email);
        *ptr_aux2=byte;
       }
      }
     }else if(subject_ativo || (!subject[0] && !cmp_str(linha, "Subject:", 8))){
      if(!subject_ativo){
       ptr_aux1=&linha[8];
       while(*ptr_aux1 == ' ')ptr_aux1++;
      }else ptr_aux1=linha;
      ptr_aux2=subject+subject_ativo;
      subject_ativo+=selor_strncpy(ptr_aux2, ptr_aux1, subject_rest);
      subject_rest-=subject_ativo;
      ptr_aux2=subject+subject_ativo;
     }else if(!cmp_str("Date:", linha, 5)){
      ptr_aux1=&linha[5];
      while(*ptr_aux1 == ' ')ptr_aux1++;
      selor_strncpy(date, ptr_aux1, MAX_tam_date);
     }
     cont2=0;
    }else if(buffer[cont1] != 13)linha[cont2++]=buffer[cont1];
   }
  }
 }else {
  if(!(arq_tmp=fopen(op_sf, "rb"))){
   if(op_v)printf("Could not open the file %s\n", op_sf);
   return 1;
  }
 }
 
 if(!date[0]){
  // Campo com a data nao foi encontrado
  // Date: Tue, 22 Jun 2010 16:00:08 -0300 (BRT)
  pegtempo();
  timeinfo=localtime(&tempo);
  snprintf(str_fuso, sizeof(str_fuso), "%+.2d00", (int)(timeinfo->tm_gmtoff/3600)+timeinfo->tm_isdst);
  snprintf(date, MAX_tam_date, "%s, %s %s %s %s:%s:%s %s", infotempo.dia_semana,infotempo.dia,infotempo.mes,infotempo.ano,infotempo.hora,infotempo.minuto,infotempo.segundo,str_fuso);
 }
 
 fseek(arq_tmp, 0, 0);
 if(!num_rcpts){
  if(op_v)printf("No recipients\n");
  return 0;
 }
 if((socket_remoto=socket(AF_INET, SOCK_STREAM, 0)) == -1){
  if(op_v)printf("Error: %s\n\n", strerror(errno));
  fclose(arq_tmp);
  remove(cam_arq_tmp);
  return 1;
 }
 if(setsockopt(socket_remoto, SOL_SOCKET, SO_REUSEADDR, &numero, sizeof(int)) == -1){
  if(op_v)printf("Error: %s\n\n", strerror(errno));
  fclose(arq_tmp);
  remove(cam_arq_tmp);
  return 1;
 }
 
 if(host[0]){
  // Resolvendo host
  if(!(hostname=gethostbyname(host)))selor_strncpy(ip, host, 16);
  else selor_strncpy(ip, inet_ntoa(*((struct in_addr *)hostname->h_addr_list[0])), 16);
 }
 rede_remota.sin_family=AF_INET;
 rede_remota.sin_port=htons(port);
 rede_remota.sin_addr.s_addr=inet_addr(ip);
 memset(rede_remota.sin_zero,0,8);
 if((connect(socket_remoto, (struct sockaddr *)&rede_remota, sizeof(struct sockaddr))) == -1){
  if(op_v)printf("\nError: %s\n", strerror(errno));
  close(socket_remoto);
  fclose(arq_tmp);
  remove(cam_arq_tmp);
  return 1;
 }
 b_gravados=selor_recv(socket_remoto, buffer, TAM_buffer, 0);
 if(b_gravados < 1 || strncmp(buffer, "220 ", 4)){
  Conexao_perdida:
  if(op_v){
   if(b_gravados > 0){
    buffer[b_gravados]=0;
    printf("Response from server: %s", buffer);
   }else printf("Connection lost: %s\n", strerror(errno));
  }
  close(socket_remoto);
  if(arq_tmp)fclose(arq_tmp);
  remove(cam_arq_tmp);
  return 1;
 }
 
 selor_send(socket_remoto, "ehlo localhost\r\n", 16, 0);
 b_gravados=selor_recv(socket_remoto, buffer, TAM_buffer, 0);
 if(b_gravados < 1 || strncmp(buffer, "250", 3))goto Conexao_perdida;
 
 
 if(user[0]){
  // Autenticando-se
  selor_send(socket_remoto, "auth login\r\n", 12, 0);
  b_gravados=selor_recv(socket_remoto, buffer, TAM_buffer, 0);
  if(b_gravados < 1 || strncmp(buffer, "334 ", 4)){
   if(op_v)printf("Autentication falied\n");
   goto Conexao_perdida;
  }
  base64_codif(linha, user, strlen(user));
  strcat(linha, "\r\n");
  cont1=strlen(linha);
  selor_send(socket_remoto, linha, cont1, 0);
  b_gravados=selor_recv(socket_remoto, buffer, TAM_buffer, 0);
  if(b_gravados < 1 || strncmp(buffer, "334 ", 4)){
   if(op_v)printf("Autentication falied\n");
   goto Conexao_perdida;
  }
  base64_codif(linha, pass, strlen(pass));
  strcat(linha, "\r\n");
  cont1=strlen(linha);
  selor_send(socket_remoto, linha, cont1, 0);
  b_gravados=selor_recv(socket_remoto, buffer, TAM_buffer, 0);
  if(b_gravados < 1 || strncmp(buffer, "235 ", 4)){
   if(op_v)printf("Autentication falied\n");
   goto Conexao_perdida;
  }
 }
 
 ret_email(from, linha);
 b_gravados=sprintf(buffer, "mail from: <%s>\r\n", linha);
 selor_send(socket_remoto, buffer, b_gravados, 0);
 b_gravados=selor_recv(socket_remoto, buffer, TAM_buffer, 0);
 if(b_gravados < 1 || strncmp(buffer, "250 ", 4))goto Conexao_perdida;
 
 for(cont1=0; cont1<num_rcpts; cont1++){
  ret_email(rcpts[cont1], linha);
  b_gravados=sprintf(buffer, "rcpt to: <%s>\r\n", linha);
  selor_send(socket_remoto, buffer, b_gravados, 0);
  b_gravados=selor_recv(socket_remoto, buffer, TAM_buffer, 0);
  if(b_gravados < 1 || strncmp(buffer, "250 ", 4))goto Conexao_perdida;
 }
 
 selor_send(socket_remoto, "data\r\n", 6, 0);
 b_gravados=selor_recv(socket_remoto, buffer, TAM_buffer, 0);
 if(b_gravados < 1 || strncmp(buffer, "354 ", 4))goto Conexao_perdida;
 cont2=0;
 header_enviado=0;
 
 boundary[0]=0;
 from_ativo=0;
 subject_ativo=0;
 date_ativo=0;
 if(op_sf[0]){
  // From
  if(from[0])cont3=snprintf(linha, TAM_buffer, "From: %s\r\n", from);
  else cont3=snprintf(linha, TAM_buffer, "From: <>\r\n");
  b_gravados=selor_send(socket_remoto, linha, cont3, 0);
  
  // To
  cont3=snprintf(linha, TAM_buffer, "To: %s\r\n", rcpts[0]);
  b_gravados=selor_send(socket_remoto, linha, cont3, 0);
  for(cont1=1; cont1<num_rcpts; cont1++){
   cont3=snprintf(linha, TAM_buffer, " %s\r\n", rcpts[cont1]);
   b_gravados=selor_send(socket_remoto, linha, cont3, 0);
  }
  // Subject
  if(subject[0]){
   cont3=snprintf(linha, TAM_buffer, "Subject: %s\r\n", subject);
   b_gravados=selor_send(socket_remoto, linha, cont3, 0);
  }
  // Date
  if(date[0]){
   cont3=snprintf(linha, TAM_buffer, "Date: %s\r\n", date);
   b_gravados=selor_send(socket_remoto, linha, cont3, 0);
  }
  if(total_anexos){
   // Criando multipart/mixed
   sprintf(boundary, "%ld_%ld.%d", tempo >> 1, tempo, getpid());
   cont3=sprintf(linha, "MIME-Version: 1.0\r\nContent-Type: multipart/mixed;\r\n\tboundary=\"%s\"\r\nX-Mailer: SELOR_mail %s\r\n", boundary, VERSAO);
   b_gravados=selor_send(socket_remoto, linha, cont3, 0);
   cont3=sprintf(linha, "\r\nThis is a multi-part message in MIME format.\r\n\r\n--%s\r\nContent-Type: text/plain;\r\n\tcharset=\"iso-8859-1\"\r\nContent-Transfer-Encoding: quoted-printable\r\n\r\n", boundary);
   b_gravados=selor_send(socket_remoto, linha, cont3, 0);
  }else b_gravados=selor_send(socket_remoto, "\r\n", 2, 0);
  header_enviado=1;
 }
 while(!feof(arq_tmp)){
  b_lidos=fread(buffer, 1, TAM_buffer, arq_tmp);
  if(!header_enviado){
   // Enviando Header Linha por Linha 
   for(cont1=0; cont1<b_lidos; cont1++){
    if(buffer[cont1] == 10){
     if(!cont2){
      // -- Fim do header. Enviando o restante do buffer --
      if(!from_ativo && from[0]){
       cont3=snprintf(linha, TAM_buffer, "From: %s\r\n", from);
       b_gravados=selor_send(socket_remoto, linha, cont3, 0);
      }
      if(!subject_ativo && subject[0]){
       cont3=snprintf(linha, TAM_buffer, "Subject: %s\r\n", subject);
       b_gravados=selor_send(socket_remoto, linha, cont3, 0);
      }
      if(!date_ativo){
       cont3=snprintf(linha, TAM_buffer, "Date: %s\r\n", date);
       b_gravados=selor_send(socket_remoto, linha, cont3, 0);
      }
      if(total_anexos){
       // Criando multipart/mixed
       sprintf(boundary, "%ld_%ld.%d", tempo >> 1, tempo, getpid());
       cont3=sprintf(linha, "MIME-Version: 1.0\r\nContent-Type: multipart/mixed;\r\n\tboundary=\"%s\"\r\nX-Mailer: SELOR_mail %s\r\n", boundary, VERSAO);
       b_gravados=selor_send(socket_remoto, linha, cont3, 0);
       cont3=sprintf(linha, "\r\nThis is a multi-part message in MIME format.\r\n\r\n--%s\r\nContent-Type: text/plain;\r\n\tcharset=\"iso-8859-1\"\r\nContent-Transfer-Encoding: quoted-printable\r\n\r\n", boundary);
       b_gravados=selor_send(socket_remoto, linha, cont3, 0);
      }
      header_enviado=1;
      buffer[b_lidos]=0;
      b_gravados=selor_send(socket_remoto, &buffer[cont1], b_lidos-cont1, 0);
      if(b_gravados != b_lidos-cont1){
       b_gravados=0;
       goto Conexao_perdida;
      }
      cont1=b_lidos;
     }else {
      linha[cont2]=0;
      if(!cmp_str(linha, "From:", 5)){
       snprintf(linha, TAM_buffer, "From: %s\r\n", from);
       from_ativo=1;
      }else if(!cmp_str(linha, "Subject:", 8)){
       snprintf(linha, TAM_buffer, "Subject: %s\r\n", subject);
       subject_ativo=1;
      }else if(!cmp_str(linha, "Date:", 5)){
       snprintf(linha, TAM_buffer, "Date: %s\r\n", date);
       date_ativo=1;
      }else strcat(linha, "\r\n");
      cont3=strlen(linha);
      linha[cont3]=0;
      b_gravados=selor_send(socket_remoto, linha, cont3, 0);
      if(b_gravados != cont3){
       b_gravados=0;
       goto Conexao_perdida;
      }
     }
     cont2=0;
    }else if(buffer[cont1] != 13)linha[cont2++]=buffer[cont1];
   }
  }else {
   buffer[b_lidos]=0;
   b_gravados=selor_send(socket_remoto, buffer, b_lidos, 0);
   if(b_gravados != b_lidos){
    b_gravados=0;
    goto Conexao_perdida;
   }
  }
 }
 fclose(arq_tmp);
 if(total_anexos && boundary[0]){
  // Enviand anexos
  for(cont1=0; cont1<total_anexos; cont1++){
   if((arq_tmp=fopen(op_a[cont1], "rb"))){
    ptr_aux1=op_a[cont1]+strlen(op_a[cont1]);
    while(ptr_aux1 > op_a[cont1] && *ptr_aux1 != '/')ptr_aux1--;
    if(*ptr_aux1 == '/')ptr_aux1++;
    b_lidos=sprintf(buffer,"\r\n\r\n--%s\r\nContent-Type: application/octet-stream;\r\n\tname=\"%s\"\r\nContent-Transfer-Encoding: base64\r\nContent-Disposition: attachment;\r\n\tfilename=\"%s\"\r\n\r\n", boundary, ptr_aux1, ptr_aux1);
    b_gravados=selor_send(socket_remoto, buffer, b_lidos, 0);
    while(!feof(arq_tmp)){
     // Encodando o anexo em base64 e em linhas de 76 bytes.
     // 57 bytes encodados em base64 ocupam 76 bytes
     b_lidos=fread(buffer, 1, 57, arq_tmp);
     base64_codif(linha, buffer, b_lidos);
     b_lidos=sprintf(buffer, "%s\r\n", linha);
     b_gravados=selor_send(socket_remoto, buffer, b_lidos, 0);
     if(b_gravados != b_lidos)goto Conexao_perdida;
    }
    fclose(arq_tmp);
   }else if(op_v)printf("The file %s could not be found\n", op_a[cont1]);
  }
  // Fim do email
  b_lidos=snprintf(linha, sizeof(linha), "--%s--\r\n", boundary);
  b_gravados=selor_send(socket_remoto, linha, b_lidos, 0);
 }
 arq_tmp=NULL;
 selor_send(socket_remoto, "\r\n.\r\n", 5, 0);
 b_gravados=selor_recv(socket_remoto, buffer, TAM_buffer, 0);
 if(b_gravados < 1 || strncmp(buffer, "250 ", 4))goto Conexao_perdida;
 selor_send(socket_remoto, "quit\r\n", 6, 0);
 close(socket_remoto);
 
 return 0;
}











